<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use CambioReal\CambioReal;

class WC_CambioReal_Api_Controller
{
	/**
	 * Construct the controller with our config object
	 */
	public function __construct()
	{
		$this->user_id = get_current_user_id();
	}

	/**
	 * Processing CambioReal status notifications.
	 *
	 * @param string $token
	 *
	 * @return void
	 */
	public function notify( $token ) {
		WC_CambioReal::log( sprintf( __( 'Notification received. Token: %s', 'wc-gateway-cambioreal' ), $token ));

		$order_id = WC_CambioReal::get_post_id_by_meta('_cr_token', $token);
		$order = new WC_Order( $order_id );

		if (! $order->get_id()) {
			$message = sprintf( __( 'Ordem não encontrada. Token: %s', 'wc-gateway-cambioreal' ), $token );
			WC_CambioReal::log( $message );

			return wp_send_json(['status' => 'error', 'message' => $message], 400);
		}

        $response = CambioReal::get([
            'token' => $token,
        ]);

		WC_CambioReal::log( sprintf( __( 'Notification processing response: %s', 'wc-gateway-cambioreal' ), print_r( $response, true ) ) );

        if (empty($response) || $response->status !== 'success') {
			return wp_send_json(['status' => 'error', 'message' => $response->message], 400);
        }

        // Order Status changes
    	WC_CambioReal::update_payment_status($order, $response->data);

		return wp_send_json(['status' => 'success']);
	}

	/**
	 * Redirect to checkout if CambioReal returns a error.
	 *
	 * @param int $order_id
	 * @return void
	 */
	public function checkout_success($order_id) {
		WC_CambioReal::log( __( 'Checkout success redirect.', 'wc-gateway-cambioreal' ) );

		$order = new WC_Order( $order_id );

		// Empty cart
		WC()->cart->empty_cart();

		$order->add_order_note( __( 'CambioReal: The transaction was created.', 'wc-gateway-cambioreal' ) );

		wp_redirect( $order->get_checkout_order_received_url() );
	}

	/**
	 * Redirect to checkout if CambioReal returns a error.
	 *
	 * @param int $order_id
	 * @return void
	 */
	public function checkout_error($order_id) {
		WC_CambioReal::log( __( 'Checkout ERROR redirect.', 'wc-gateway-cambioreal' ) );

		$order = new WC_Order( $order_id );
		$order->update_status( 'cancelled', __( 'CambioReal: Cancelled by checkout error.', 'wc-gateway-cambioreal' ) );

		$this->restore_cart();

		wc_add_notice('Oops! Ocorreu um erro, entre em contato conosco ou tente novamente mais tarde.', 'error');
		wp_redirect( wc_get_checkout_url() );
	}

	/**
	 * Restore the items of the cart after redirect.
	 *
	 * @return void
	 */
	public function restore_cart() {
		// delete current cart.
		WC()->cart->empty_cart( true );

		// update user meta with saved persistent.
		$saved_cart = get_user_meta( $this->user_id, '_cambioreal_persistent_cart', true );

		// then reload cart.
		WC()->session->set( 'cart', $saved_cart );
		WC()->cart->get_cart_from_session();

		// Remove user meta
		delete_user_meta($this->user_id, '_cambioreal_persistent_cart');
	}
}
