<?php

/**
 * @author      CambioReal
 * @copyright   2024 CambioReal (https://www.cambioreal.com)
 * @license     https://www.cambioreal.com Copyright
 *
 * @link        https://www.cambioreal.com
 */

use CambioReal\CambioReal;

defined('ABSPATH') || exit;

if (!function_exists('add_action')) {
    exit(0);
}

if (! class_exists('WC_Payment_Gateway')) {
    wp_die('Sorry, but this plugin requires the Woocommerce plugin to be installed and active.', null, ['back_link' => true]);
}

/**
 * Abstract Gateway
 * @package WooCommerce/Classes/Payment
 */
abstract class WC_Abstract_Gateway_CambioReal extends WC_Payment_Gateway
{

    /** @var bool Whether or not logging is enabled */
    public static $log_enabled = false;

    protected $app_id;
    protected $app_secret;
    protected $test_mode;
    protected $debug_enabled;
    protected $duplicate;
    protected $take_rates;
    protected $show_rate;

    /**
     * Constructor for the gateway.
     */
    public function __construct()
    {
        $this->has_fields = false;

        // Load the settings.
        $this->init_form_fields();
        $this->init_settings();

        // Define user set variables
        $this->setup_configs();

        // Actions
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);
    }

    /**
     * Sets up the configuration object
     *
     * @return void
     */
    public function setup_configs()
    {
        // Define user set variables
        $this->app_id = $this->get_option('app-id');
        $this->app_secret = $this->get_option('app-secret');
        $this->test_mode = $this->get_option('test-mode', 'no') === 'yes';
        $this->debug_enabled = $this->get_option('debug-enabled');

        $this->title = $this->get_option('title');
        $this->duplicate = $this->get_option('duplicate', 'no') === 'yes';
        $this->take_rates = $this->get_option('take-rates', 'no') === 'yes';
        $this->show_rate = $this->get_option('show-rate', 'no') === 'yes';

        \CambioReal\Config::set([
            'appId'     => $this->app_id,
            'appSecret' => $this->app_secret,
            'testMode'  => $this->test_mode,
        ]);
    }

    protected function simulate()
    {
        if (! $this->show_rate) {
            return false;
        }

        try {
            /**
             * Request para realizar a simulação de valores.
             */
            $response = CambioReal::simulator([
                'currency'       => get_woocommerce_currency(),
                'amount'         => WC()->cart->total,
                'take_rates'     => $this->take_rates,
                'payment_method' => 'boleto',
            ]);

            if (! empty($response) && $response->status === 'success') {
                return $response->data;
            }

        } catch (Exception $e) {
            WC_CambioReal::log($e->getMessage(), $this);
        }
    }
}
