<?php

use CambioReal\CambioReal;

/**
 * Bank Transfer Payment Gateway.
 *
 * Provides a Bank Transfer Payment Gateway. Based on code by Mike Pepper.
 *
 * @version     1.0.0
 * @package     WooCommerce/Classes/Payment
 * @author      CambioReal
 */
class WC_Gateway_Checkout_CambioReal extends WC_Abstract_Gateway_CambioReal
{
    /**
     * Constructor for the gateway.
     */
    public function __construct()
    {
        $this->id = 'wc-cambioreal-checkout';
        $this->method_title = __('CambioReal - Checkout', 'woocommerce');
        $this->method_description = __('CambioCheckout sends customers to CambioReal to enter their personal information for make the payment.', 'woocommerce');

        parent::__construct();
    }

    /**
     * Get gateway icon.
     * @return string
     */
    public function get_description()
    {
        $icon = 'https://www.cambioreal.com/img/plugins/woocommerce/payment-methods.png';

        $description_html = '<img style="max-width: 100%;height: auto;margin: auto;" src="' . esc_attr($icon) . '" alt="' . esc_attr__('CambioCheckout', 'woocommerce') . '" />';
        $description_html .= '<p style="clear: both;line-height: 1.2;margin: 5px 0 10px;"><small>';

        if ($simulation = $this->simulate()) {
            $description_html .= __('Taxa de câmbio', 'wc-gateway-cambioreal') . ': <strong>R$ ' . number_format($simulation->rate, 4) . '</strong>';
            $description_html .= '<br>' . __('Total a pagar com IOF', 'wc-gateway-cambioreal') . ' (' . $simulation->iof_percentage . '%): <strong>R$ ' . (($simulation->currency === 'BRL') ? number_format($simulation->amount, 2) : number_format($simulation->result, 2)) . '</strong>';
        }

        $description_html .= '</small></p>';

        return apply_filters('woocommerce_gateway_description', $description_html, $this->id);
    }

    /**
     * Initialise Gateway Settings Form Fields.
     */
    public function init_form_fields()
    {
        $this->form_fields = [
            // Configurações de integração.
            'integration'   => [
                'title'       => __('Integration', 'wc-gateway-cambioreal'),
                'description' => sprintf(__('Enter your CambioReal API credentials. You can obtain the credentials in the direct API integration section, access the CambioReal panel by clicking <a href="%s" target="_blank">here</a>.', 'wc-gateway-cambioreal'), 'https://business.cambioreal.com/en/panel/integration'),
                'type'        => 'title',
            ],
            'enabled'       => [
                'title'   => __('Enable/Disable', 'woocommerce'),
                'label'   => __('Enable CambioCheckout', 'woocommerce'),
                'type'    => 'checkbox',
                'default' => 'no',
            ],
            'app-id'        => [
                'title'       => __('App Id', 'wc-gateway-cambioreal'),
                'description' => __('To get this information go to the CambioReal website and enter your Company Panel > Integration > Magento', 'wc-gateway-cambioreal'),
                'default'     => '',
                'type'        => 'text',
                'desc_tip'    => true,
            ],
            'app-secret'    => [
                'title'       => __('App Secret', 'wc-gateway-cambioreal'),
                'description' => __('You will receive this information along with the App Id.', 'wc-gateway-cambioreal'),
                'default'     => '',
                'type'        => 'password',
                'desc_tip'    => true,
            ],
            'test-mode'     => [
                'title'       => __('CambioReal Sandbox', 'wc-gateway-cambioreal'),
                'label'       => __('Enable CambioReal sandbox', 'wc-gateway-cambioreal'),
                'description' => sprintf(__('CambioReal sandbox can be used to test payments. Sign up for a company account <a href="%s" target="_blank">here</a>.', 'wc-gateway-cambioreal'), 'https://sandbox.cambioreal.com/br/receber'),
                'default'     => 'no',
                'type'        => 'checkbox',
                'desc_tip'    => true,
            ],
            'debug-enabled' => [
                'title'       => __('Debug Log', 'wc-gateway-cambioreal'),
                'label'       => __('Enable Debug Log', 'wc-gateway-cambioreal'),
                'description' => __('Record all errors that occur when executing a transaction.', 'wc-gateway-cambioreal'),
                'default'     => 'no',
                'type'        => 'checkbox',
                'desc_tip'    => true,
            ],

            // Configurações de visualização.
            'method-details' => [
                'title'       => __('Payment method details', 'wc-gateway-cambioreal'),
                'description' => __('Customization of how the payment method name is displayed.', 'wc-gateway-cambioreal'),
                'type'        => 'title',
            ],
            'title'          => [
                'title'       => __('Title', 'woocommerce'),
                'description' => __('This controls the title which the user sees during checkout.', 'woocommerce'),
                'default'     => __('CambioCheckout', 'woocommerce'),
                'type'        => 'text',
                'desc_tip'    => true,
            ],

            // Configurações do método de pagamento.
            'advanced'   => [
                'title'       => __('Payment Options', 'wc-gateway-cambioreal'),
                'description' => __('Advanced payment method options.', 'wc-gateway-cambioreal'),
                'type'        => 'title',
            ],
            'duplicate'  => [
                'title'       => __('Duplicate', 'wc-gateway-cambioreal'),
                'label'       => __('Allow to generate another transaction after expiration', 'wc-gateway-cambioreal'),
                'description' => __('Allow to generate another transaction after expiration without the customer having to create another order in the system.', 'wc-gateway-cambioreal'),
                'default'     => 'no',
                'type'        => 'checkbox',
                'desc_tip'    => true,
            ],
            'take-rates' => [
                'title'       => __('Take Rates', 'wc-gateway-cambioreal'),
                'label'       => __('I want to be responsible for all fees', 'wc-gateway-cambioreal'),
                'description' => __('No fees will be charged from the customer (includes iof). They will be discounted from the business final payment.', 'wc-gateway-cambioreal'),
                'default'     => 'no',
                'type'        => 'checkbox',
                'desc_tip'    => true,
            ],
            'show-rate'  => [
                'title'       => __('Show cart simulation', 'wc-gateway-cambioreal'),
                'label'       => __('Enable cart simulation', 'wc-gateway-cambioreal'),
                'description' => __('This option shows in the cart the exchange rate and the value with IOF to be paid by the customer at the checkout of CambioReal.', 'wc-gateway-cambioreal'),
                'default'     => 'no',
                'type'        => 'checkbox',
                'desc_tip'    => true,
            ],
        ];
    }

    /**
     * Process the payment and return the result.
     *
     * @param int $order_id
     * @return array
     */
    public function process_payment($order_id)
    {
        $order = wc_get_order($order_id);

        $items = $order->get_items();
        $products = [];
        $total = 0;

        // Products
        foreach ($items as $item) {
            $product = wc_get_product($item['product_id']);
            $terms = wp_get_post_terms($item['product_id'], 'product_cat', ['fields' => 'names']);
            $brands = wp_get_post_terms($item['product_id'], 'product_brand', ['fields' => 'names']);

            if (empty($brands) || $brands instanceof WP_Error) {
                $brands = $product->get_attribute('brand') ?: $product->get_attribute('marca');
                $brands = array_filter([$brands]);
            }

            $products[] = [
                'descricao'  => $product->get_title(),
                'ref'        => $product->get_id(),
                'base_value' => $item['line_subtotal'],
                'valor'      => $item['line_total'],
                'qty'        => $item['qty'],
                'category'   => implode(',', $terms),
                'brand'      => implode(',', $brands),
                'sku'        => $product->get_sku(),
            ];

            $total += $item['line_total'];
        }

        // Shipping Tax
        if ($shipping = $order->get_shipping_total()) {
            $products[] = [
                'descricao'  => $order->get_shipping_method(),
                'ref'        => 'shipping',
                'base_value' => $shipping,
                'valor'      => $shipping,
            ];

            $total += $shipping;
        }

        // Fix to adjust cart price when users use a custom discount plugin for WooCommerce.
        if ($total != $order->get_total()) {
            $adjustment = $order->get_total() - $total;

            $products[] = [
                'descricao'  => 'WooCommerce Total Adjustment',
                'ref'        => 'adjustment',
                'base_value' => $adjustment,
                'valor'      => $adjustment,
            ];
        }

        $params = [
            'client'       => [
                'name'  => $order->get_formatted_billing_full_name(),
                'email' => $order->billing_email,
            ],
            'currency'     => get_woocommerce_currency(),
            'amount'       => $order->get_total(),
            'order_id'     => $order_id,
            'duplicate'    => $this->duplicate,
            'take_rates'   => $this->take_rates,
            'products'     => $products,
            'url_callback' => $this->get_return_url($order),
            'url_callback' => get_site_url() . '/cambioreal/payment/checkout-success/?order_id=' . $order_id,
            'url_error'    => get_site_url() . '/cambioreal/payment/checkout-error/?order_id=' . $order_id,
        ];

        try {
            // Create solicitation
            $response = CambioReal::request($params);

            return $this->process_response($response, $order);
        } catch (Exception $e) {
            WC_CambioReal::log($e->getMessage(), $this);
        }

        wc_add_notice('Oops! Ocorreu um erro, entre em contato conosco ou tente novamente mais tarde.', 'error');
    }

    /**
     *
     * @param StdClass $response
     * @param WC_Order $order
     *
     * @throws Exception Throws error message.
     * @throws Exception Throws parameter missing exception.
     */
    protected function process_response($response, $order)
    {
        WC_CambioReal::log(sprintf(__('Processing response: %s', 'wc-gateway-cambioreal'), print_r($response, true)), $this);

        if ($response->status !== 'success') {
            $this->process_response_error($response, $order);
        }

        $message = sprintf(__('Transaction created. Token: %s', 'wc-gateway-cambioreal'), $response->data->token);

        WC_CambioReal::log($message, $this);

        // Mark as on-hold (we're awaiting the payment)
        // $order->update_status('on-hold', __('Awaiting CambioReal payment.', 'woocommerce'));

        // Save the cart
        update_user_meta(get_current_user_id(), '_cambioreal_persistent_cart', WC()->cart->get_cart());

        // Reduce stock levels
        wc_reduce_stock_levels($order->get_id());

        // Empty cart
        WC()->cart->empty_cart();

        $order->add_order_note(__("CambioReal: The request {$response->data->code} was created.", 'wc-gateway-cambioreal'));

        // Save post's meta fields.
        $this->save_order_meta_fields($order, $response->data);

        return [
            'result'   => 'success',
            'redirect' => $response->data->checkout,
        ];
    }

    /**
     *
     * @param StdClass $response
     * @param WC_Order $order
     *
     * @throws Exception Throws error message.
     * @throws Exception Throws parameter missing exception.
     */
    protected function process_response_error($response, $order)
    {
        $error_message = sprintf(__('CambioReal: An error occurred: %s', 'wc-gateway-cambioreal'), $response->message);

        $order->update_status('failed', $error_message);
        $order->add_order_note($error_message);

        throw new Exception($response->message);
    }

    /**
     * Save order's meta fields for future use
     *
     * @param  WC_Order $order The order created.
     * @param  Object   $data The data from CambioReal success response.
     * @return void
     */
    protected function save_order_meta_fields($order, $data)
    {
        update_post_meta($order->get_id(), '_cr_token', $data->token);
        update_post_meta($order->get_id(), '_cr_ticket', $data->checkout);
    }

}
