<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class WC_CambioReal_Query_Router {
	protected $key = null;
	protected $routes = array();

	public function __construct( $key ) {
		self::validate_key( $key );
		$this->key = $key;
	}

	public function map( $key_value, $handler ) {
		self::validate_key_value( $key_value );
		self::validate_route( $handler );

		$this->routes[ $key_value ] = $handler;
	}

	public function serve() {
		$route = $this->route();

		if (! $route) {
			return;
		}

		$params = self::params($route);

		call_user_func_array( $route, $params );
		exit;
	}

	protected function route( $search_key = null ) {
		$method = WC_CambioReal_Request::method($this->key);

		if ( ! $method ) {
			return null;
		}

		if ( ! isset( $this->routes[ $method ] ) ) {
			return null;
		}

		return $this->routes[ $method ];
	}

	private static function params( $route ) {
		self::validate_route( $route );

		$ref  = new ReflectionMethod( $route[0], $route[1] );
		$list = $ref->getParameters();

		$params = [];
		foreach ( $list as $arg ) {
			$name = $arg->name;
			if ( ! WC_CambioReal_Request::has( $name ) ) {
				$params[ $name ] = null;
				continue;
			}

			$params[ $name ] = WC_CambioReal_Request::get( $name );
		}

		return $params;
	}

	protected static function validate_key( $subject ) {
		if ( ! is_string( $subject ) ) {
			throw new \InvalidArgumentException( 'A query route key must be a string!' );
		}
	}

	protected static function validate_key_value( $subject ) {
		if ( ! is_string( $subject ) ) {
			throw new \InvalidArgumentException( 'A query route key value must be a string!' );
		}
	}

	protected static function validate_route( $subject ) {
		if ( ! is_callable( $subject ) ) {
			throw new \InvalidArgumentException( 'Specified route is not callable!' );
		}
	}
}
