<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class WC_CambioReal_Request {
	/**
	 * Reads _REQUEST and gets the value from '$param'
	 *
	 * @param  string $param   The key from _REQUEST.
	 * @param  mixed  $default What you want to return if there's no $_REQUEST[$param].
	 * @return mixed          $_REQUEST[$param] value OR default OR
	 */
	public static function get( $param = null, $default = null ) {
		if (! $param) {
			return $_REQUEST;
		}

		if (! self::has( $param ) || ! isset( $_REQUEST[ $param ] )) {
			return $default;
		}

		return $_REQUEST[ $param ];
	}

	/**
	 * Sets $value in $_REQUEST[$param]
	 *
	 * @param string $param The key from _REQUEST.
	 * @param mixed  $value The value you want to set.
	 * @return void
	 */
	public static function set( $param, $value ) {
		$_REQUEST[ $param ] = $value;
	}

	/**
	 * Check if a key is set on $_REQUEST
	 *
	 * @param string $key The key you want to check.
	 *
	 * @return boolean        True if $_REQUEST has $key key.
	 */
	public static function has( $key ) {
		// @codingStandardsIgnoreLine
		return array_key_exists( $key, $_REQUEST );
	}

	/**
	 * Checks if current url is $path.
	 * Necessary for routers.
	 *
	 * @return boolean
	 */
	public static function method($base_url) {
		$segments = array_diff(static::segments(), static::segments($base_url));

		if (! empty($segments)) {
			return array_shift($segments);
		}
	}

    /**
     * Get the current path info for the request.
     *
     * @return string
     */
    public static function path() {
        $pattern = trim(rawurldecode($_SERVER['REQUEST_URI']), '/');

        return $pattern == '' ? '/' : $pattern;
    }

    /**
     * Get all of the segments for the request path.
     *
     * @return array
     */
    public static function segments($path = null) {
    	if (! $path) {
    		$path = static::path();
    	}

        $segments = explode('/', $path);

        return array_values(array_filter($segments, function ($value) {
            return $value !== '';
        }));
    }
}
